""" Copyright start
  Copyright (C) 2008 - 2025 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """

import os
import re
import sys
import json
import time
from constants import *
from helpers.logger import Logger
from datetime import date,datetime
from helpers.utils import Utilities
from framework.utility import messages
from helpers.cmd_utils import CmdUtils
from framework.utility.config import Config
from framework.utility.task_utils import TaskUtilities
from helpers.task_file_generator import TaskFileGenerator
from helpers.shell_file_generator import ShellFileGenerator
from framework.utility.dual_output_handler import DualOutputHandler

class Framework(object):
    def __init__(self) -> None:
        self.cmd_line_utilities = CmdUtils()
        self.task_utilities = TaskUtilities()
        self.utilities = Utilities()
        self.number_of_args = len(sys.argv)
        self.current_version = TaskUtilities.get_current_version()
        self.logger = Logger.get_logger(__name__)
        self.config = Config.get_config()
        
    def print_data(self, phase, phase_tasks):
        unregistered_tasks = phase_tasks["unregistered_tasks"] if "unregistered_tasks" in phase_tasks else []
        sorted_dict_of_phase_tasks = phase_tasks["sorted_dict_of_phase_tasks"] if "sorted_dict_of_phase_tasks" in phase_tasks else {}
        if unregistered_tasks:
            print(messages.UNREGISTERED_TASKS.format(""))
            print(f"{unregistered_tasks}\n")
        if sorted_dict_of_phase_tasks:
            print(messages.LIST_OF_TASKS.format(phase))
            for index, task in sorted_dict_of_phase_tasks.items():
                print("{:<4}{:<50}".format(str(index)+'.',task))

    def _function_code(self, version, phase, operation, dry_run=False, skip_task_download=False, status_file=None, task_name=None):
        try:
            phase_name = PHASE_MAP_DICT[phase]
            phase_tasks = {"sorted_dict_of_phase_tasks": {}, "unregistered_tasks": []}
            
            if not skip_task_download:
                self.task_utilities.download_tasks_from_repo(version)

            sorted_dict_of_phase_tasks, unregistered_tasks = self.task_utilities.identify_tasks(
                phase_name)
            if unregistered_tasks:
                phase_tasks["unregistered_tasks"] = unregistered_tasks
                if operation == 'execute':
                    if task_name in unregistered_tasks:
                        msg = messages.UNREGISTERED_TASKS.format([task_name])
                    else:
                        msg = messages.UNREGISTERED_TASKS.format(unregistered_tasks)
                    self.logger.error(msg)
                    self.task_utilities.print_justified_text(msg)
                    print(unregistered_tasks)
                    sys.exit()
            if sorted_dict_of_phase_tasks:
                if operation == 'list':
                    phase_tasks["sorted_dict_of_phase_tasks"] = sorted_dict_of_phase_tasks
                    return phase_tasks

                # Following code will create report-<version>.json & step_result-<version>.json files if not present with metadata in it.
                # If present, will update metadata in both files.
                current_datetime = datetime.utcnow()
                current_datetime = current_datetime.strftime("%d/%m/%Y, %H:%M:%S")
                metadata = {"Current FortiSOAR version" : f"{self.current_version}"}
                file_data={}
                if not os.path.isfile(REPORT_FILE.format(version)):
                    with open(REPORT_FILE.format(version), 'w') as file:
                        metadata["File creation time"] = f"{current_datetime}"
                        metadata["File modification time"] = ""
                        file_data["metadata"] = metadata
                        file.write(json.dumps(file_data))
                    os.system(
                        f"chown {FSR_ELEVATE_USER}:{FSR_ELEVATE_USER} {REPORT_FILE.format(version)}")
                    os.system(f"chmod 644 {REPORT_FILE.format(version)}")
                else:
                    with open(REPORT_FILE.format(version), 'r') as file:
                        file_data = json.load(file)
                    
                    with open(REPORT_FILE.format(version), 'w') as file:
                        file_data["metadata"]["Current FortiSOAR version"] = f"{self.current_version}"
                        file_data["metadata"]["File modification time"] = f"{current_datetime}"
                        file_data = json.dumps(file_data, indent=4, separators=(',', ': '))
                        file.write(file_data)

                if not os.path.isfile(STEP_RESULT_FILE.format(version)):
                    with open(STEP_RESULT_FILE.format(version), 'w') as file:
                        metadata["File creation time"] = current_datetime
                        metadata["File modification time"] = ""
                        file_data["metadata"] = metadata
                        file.write(json.dumps(file_data))
                    os.system(
                        f"chown {FSR_ELEVATE_USER}:{FSR_ELEVATE_USER} {STEP_RESULT_FILE.format(version)}")
                    os.system(f"chmod 644 {STEP_RESULT_FILE.format(version)}")
                else:
                    with open(STEP_RESULT_FILE.format(version), 'r') as file:
                        file_data = json.load(file)
                    
                    with open(STEP_RESULT_FILE.format(version), 'w') as file:
                        file_data["metadata"]["Current FortiSOAR version"] = f"{self.current_version}"
                        file_data["metadata"]["File modification time"] = f"{current_datetime}"
                        file_data = json.dumps(file_data, indent=4, separators=(',', ': '))
                        file.write(file_data)

                if task_name:
                    phase_tasks = list(sorted_dict_of_phase_tasks.values())
                    if task_name.endswith('.py'):
                        task_name = task_name.replace('.py','')
                    if task_name not in phase_tasks and task_name not in unregistered_tasks:
                        self.task_utilities.print_justified_text(messages.TASK_NOT_FOUND.format(phase_name, task_name, f"{FRAMEWORK_PATH}/workflow/{phase_name}"))
                        sys.exit()
                    else:
                        id = list(sorted_dict_of_phase_tasks.keys())[phase_tasks.index(task_name)]
                        sorted_dict_of_phase_tasks = {id : task_name}
                
                # This will execute all task files under provided phase
                self.task_utilities.execute_tasks(sorted_dict_of_phase_tasks, phase_name, dry_run, status_file)
                return True
            else:
                self.task_utilities.print_justified_text(
                    "No tasks in {} phase.".format(phase))
        except KeyError as ke:
            self.logger.exception("Key error occurred : {}".format(ke))
            self.task_utilities.print_justified_text(
                messages.PROVIDE_PHASE_FROM_LIST)
            self.list_phases()
        except Exception as ex:
            self.logger.exception(ex)

    def check_readiness(self, version):
        try:
            current_process_pid = os.getpid()
            # If version is provided, run following version checks
            # Running following checks for version before validating it because of following reasons =>
            #   1] If we validate version first then it will break incase of invalid version pattern is provided
            #   2] Also no need to validate version if provided version is same or lower than current version
            if version:
                # Check provided version is as per pattern
                Utilities._check_version_for_pattern(version)
                # Compare target_version with current_version
                result = self.utilities.version_compare(self.current_version, version)
                if result == 0:
                    # Target version is same as current version
                    msg = messages.LOWER_OR_SAME_VERSION_PROVIDED.format(self.current_version, version, "same as")
                    self.logger.error("ERROR: {}".format(msg))
                    self.task_utilities.print_justified_text(msg)
                    sys.exit()
                if result == 1:
                    # Target version is lower than current version
                    msg = messages.LOWER_OR_SAME_VERSION_PROVIDED.format(self.current_version, version, "less than")
                    self.logger.error("ERROR: {}".format(msg))
                    self.task_utilities.print_justified_text(msg)
                    sys.exit()
            # Validate target_version is supported for upgrade from current_version
            version=Utilities.validate_upgrade_version(self.current_version,version)

            result = self.utilities.is_command_running(
                'check-readiness', CHECK_READINESS_STATUS_FILE)
            if result[0]:
                err_msg = messages.ERROR_OCCURED.format(
                    "execution of check-readiness command", messages.COMMAND_RUNNING.format(result[1]))
                self.logger.error(err_msg)
                sys.exit(err_msg)

            self.utilities.update_command_status(version, current_process_pid, None, None, 'started', CHECK_READINESS_STATUS_FILE)

            for phase in CHECK_READINESS_PHASES:
                result = self._function_code(
                    version, phase, 'execute', True, False, CHECK_READINESS_STATUS_FILE)

            self.utilities.update_command_status(version, current_process_pid, None, None, 'ended', CHECK_READINESS_STATUS_FILE)

            if not result:
                sys.exit()
            
            report_file = REPORT_FILE.format(version)
            is_system_upgrade_ready = True
            if result and os.path.isfile(report_file):
                json_object_data = None
                try:
                    with open(report_file, 'r') as json_file:
                        json_object_data = json.load(json_file)
                except Exception as exp:
                    self.logger.exception(exp)
                    is_system_upgrade_ready = False
                if json_object_data:
                    for key, value in json_object_data.items():
                        if "result" in value and not value["result"]:
                            is_system_upgrade_ready = False
                            break
                # {TEXT_COLOR['YELLOW']}{TEXT_DECORATION['BLINK']} This code will display message 'Note:' in yellow color and blinking on cli.
                print(f"{TEXT_COLOR['YELLOW']}{TEXT_DECORATION['BLINK']}Note:{TEXT_DECORATION['RESET']}")
                print(messages.REPORT_GENERATED.format(f'report-{version}.json', f"{FRAMEWORK_PATH}/outputs/"))
                if is_system_upgrade_ready:
                    print(f"{TEXT_COLOR['GREEN']}{TEXT_DECORATION['BOLD']}SYSTEM IS READY FOR UPGRADE{TEXT_DECORATION['RESET']}")
        except Exception as ex:
            self.logger.exception(
                messages.ERROR_OCCURED.format("check readiness", ex))
            self.utilities.update_command_status(version, current_process_pid, None, None, 'ended', CHECK_READINESS_STATUS_FILE)
            sys.exit(messages.REFER_LOG)

    def list_phases(self):
        self.task_utilities.print_justified_text("List of phases :")
        for phase in LIST_PHASES:
            print("{:<5}{:<1}{:<10}".format("", "- ", phase))

    def list_phase_tasks(self, version, phase):
        try:
            version=Utilities.validate_upgrade_version(self.current_version,version)
            result = self._function_code(version, phase, 'list')
            # "result" will contain dictionary of phase tasks if supported FortiSOAR version is provided, else it will have boolean value false.
            if isinstance(result, bool) and not result:
                sys.exit()
            self.print_data(phase, result)
        except Exception as ex:
            self.logger.exception(messages.ERROR_OCCURED.format(
                "listing phase tasks", ex))
            sys.exit(messages.REFER_LOG)

    def execute_task(self, version, phase, task_name, skip_task_download):
        try:
            # If version is provided, run following version checks
            # Running following checks for version before validating it because of following reasons =>
            #   1] If we validate version first then it will break incase of invalid version pattern is provided
            #   2] Also no need to validate version if provided version is same or lower than current version
            if version:
                # Check provided version is as per pattern
                Utilities._check_version_for_pattern(version)
                # Compare target_version with current_version
                result = self.utilities.version_compare(self.current_version, version)
                if result == 1 and phase in PHASE_MAP_DICT.keys():
                    # Target version is lower than current version
                    msg = messages.LOWER_OR_SAME_VERSION_PROVIDED.format(self.current_version, version, "less than")
                    self.logger.error("ERROR: {}".format(msg))
                    self.task_utilities.print_justified_text(msg)
                    sys.exit()
            
            current_version = int(self.current_version.replace('.',''))
            target_version = int(version.replace('.',''))
            # If task is from post-upgrade phase and provided version is same as current then set target_version with current_version
            if phase == 'post-upgrade' and target_version == current_version:
                Utilities._set_target_version(version)
            # Validate provided version if it is not same as current version or task is not from post-upgrade phase
            else:
                version=Utilities.validate_upgrade_version(self.current_version,version)
                
            if phase == 'post-upgrade':
                if os.path.exists(f"/opt/fsr-elevate/elevate/outputs/step_result-{version}.json"):
                    with open(f"/opt/fsr-elevate/elevate/outputs/step_result-{version}.json", 'r') as file:
                        step_result_data = json.load(file)
                        if step_result_data == {}:
                            msg = "{}\n{}".format(messages.STEP_RESULT_EMPTY.format(version),messages.EXECUTE_PRE_UPGRADE_PHASE.format(version))
                            self.logger.error(messages.STEP_RESULT_EMPTY.format(version))
                            self.task_utilities.print_justified_text(msg)
                            sys.exit()
                else:
                    msg = "{}\n{}".format(messages.STEP_RESULT_NOT_EXISTS.format(version),messages.EXECUTE_PRE_UPGRADE_PHASE.format(version))
                    self.logger.error(messages.STEP_RESULT_NOT_EXISTS.format(version))
                    self.task_utilities.print_justified_text(msg)
                    sys.exit()
            
            result = self._function_code(version, phase, 'execute', False, skip_task_download, None, task_name)

            if not result:
                sys.exit()
        except Exception as ex:
            msg_sub_string = "executing {} task from the {} phase".format(task_name,phase)
            self.logger.exception(
                messages.ERROR_OCCURED.format(msg_sub_string, ex))
            sys.exit(messages.REFER_LOG)

    def execute_phase(self, version, phase, skip_task_download):
        try:
            current_process_pid = os.getpid()
            executing_phase_status_file = EXECUTE_PRE_UPGRADE_PHASE_STATUS_FILE if phase == 'pre-upgrade' else EXECUTE_POST_UPGRADE_PHASE_STATUS_FILE
            
            # If version is provided, run following version checks
            # Running following checks for version before validating it because of following reasons =>
            #   1] If we validate version first then it will break incase of invalid version pattern is provided
            #   2] Also no need to validate version if provided version is same or lower than current version
            if version:
                # Check provided version is as per pattern
                Utilities._check_version_for_pattern(version)
                # Compare target_version with current_version
                result = self.utilities.version_compare(self.current_version, version)
                # For pre-upgrade phase, providing current_version as target_version is not allowed
                if result == 0 and phase == 'pre-upgrade':
                    # Target version is same as current version
                    msg = messages.LOWER_OR_SAME_VERSION_PROVIDED.format(self.current_version, version, "same as")
                    self.logger.error("ERROR: {}".format(msg))
                    self.task_utilities.print_justified_text(msg)
                    sys.exit()
                # For pre-upgrade & post-upgrade phase, providing target_version same as current_version or lower than it is not allowed
                if result == 1 and phase in PHASE_MAP_DICT.keys():
                    # Target version is lower than current version
                    msg = messages.LOWER_OR_SAME_VERSION_PROVIDED.format(self.current_version, version, "less than")
                    self.logger.error("ERROR: {}".format(msg))
                    self.task_utilities.print_justified_text(msg)
                    sys.exit()
            
            current_version = int(self.current_version.replace('.',''))
            target_version = int(version.replace('.',''))
            # Validate provided version if it is not same as current version or executing phase is not post-upgrade phase
            if phase != 'post-upgrade' and target_version != current_version:
                # Validate target_version is supported for upgrade from current_version
                version=Utilities.validate_upgrade_version(self.current_version,version)
            # If post-upgrade phase is executed and provided version is same as current then set target_version with current_version
            else:
                Utilities._set_target_version(version)
            
            pre_upgrade_phase_result = self.utilities.is_command_running(
                'execute-phase', EXECUTE_PRE_UPGRADE_PHASE_STATUS_FILE)
            post_upgrade_phase_result = self.utilities.is_command_running(
                'execute-phase', EXECUTE_POST_UPGRADE_PHASE_STATUS_FILE)
            if (pre_upgrade_phase_result and pre_upgrade_phase_result[0]) or (post_upgrade_phase_result and post_upgrade_phase_result[0]):
                err_msg = messages.ERROR_OCCURED.format(
                    "execution of execute-phase command", messages.COMMAND_RUNNING.format('execute-phase'))
                self.logger.error(err_msg)
                sys.exit(err_msg)
            
            if phase == 'pre-upgrade':
                status_task = None
                status_phase = phase
                if os.path.isfile(EXECUTE_PRE_UPGRADE_PHASE_STATUS_FILE):
                    status_data = self.utilities.read_status_file(EXECUTE_PRE_UPGRADE_PHASE_STATUS_FILE)
                    if status_data:
                        status_data_task = status_data.get('task',None)
                        status_data_phase = status_data.get('phase',None)
                        status_data_status = status_data.get('status',None)
                        status_data_version = status_data.get('version',None)
                        if status_data_version == version:
                            # This condition will ensure resume of any command for same target version in status file
                            if phase == status_data_phase and status_data_task and status_data_status == 'started':
                                status_task = status_data['task']
                                status_phase = status_data['phase']
                
                self.utilities.update_command_status(version, current_process_pid, status_phase, status_task, 'started', EXECUTE_PRE_UPGRADE_PHASE_STATUS_FILE)

            if phase == 'post-upgrade':
                # Target version is greater than current version
                if target_version > current_version:
                    self.task_utilities.print_justified_text(messages.POST_UPGRADE_EXECUTION_RESTRICTION.format(version,self.current_version))
                    sys.exit()
                if os.path.exists(f"/opt/fsr-elevate/elevate/outputs/step_result-{version}.json"):
                    with open(f"/opt/fsr-elevate/elevate/outputs/step_result-{version}.json", 'r') as file:
                        step_result_data = json.load(file)
                        if step_result_data == {}:
                            msg = "{}\n{}".format(messages.STEP_RESULT_EMPTY.format(version),messages.EXECUTE_PRE_UPGRADE_PHASE.format(version))
                            self.logger.error(messages.STEP_RESULT_EMPTY.format(version))
                            self.task_utilities.print_justified_text(msg)
                            sys.exit()
                else:
                    msg = "{}\n{}".format(messages.STEP_RESULT_NOT_EXISTS.format(version),messages.EXECUTE_PRE_UPGRADE_PHASE.format(version))
                    self.logger.error(messages.STEP_RESULT_NOT_EXISTS.format(version))
                    self.task_utilities.print_justified_text(msg)
                    sys.exit()
            
                status_task = None
                status_phase = phase
                if os.path.isfile(EXECUTE_POST_UPGRADE_PHASE_STATUS_FILE):
                    status_data = self.utilities.read_status_file(EXECUTE_POST_UPGRADE_PHASE_STATUS_FILE)
                    if status_data:
                        status_data_task = status_data.get('task',None)
                        status_data_phase = status_data.get('phase',None)
                        status_data_status = status_data.get('status',None)
                        status_data_version = status_data.get('version',None)
                        if status_data_version == version:
                            # This condition will ensure resume of any command for same target version in status file
                            if phase == status_data_phase and status_data_task and status_data_status == 'started':
                                status_task = status_data['task']
                                status_phase = status_data['phase']
            
                self.utilities.update_command_status(version, current_process_pid, status_phase, status_task, 'started', EXECUTE_POST_UPGRADE_PHASE_STATUS_FILE)
            
            result = self._function_code(version, phase, 'execute', False, skip_task_download, executing_phase_status_file)

            self.utilities.update_command_status(version, current_process_pid, None, None, 'ended', executing_phase_status_file)

            if not result:
                sys.exit()
        except Exception as ex:
            msg_sub_string = "executing {} phase".format(phase)
            self.logger.exception(
                messages.ERROR_OCCURED.format(msg_sub_string, ex))
            self.utilities.update_command_status(version, current_process_pid, None, None, 'ended', executing_phase_status_file)
            sys.exit(messages.REFER_LOG)

    def execute(self, version, download_packages,local_download_directory):
        try:
            current_process_pid = os.getpid()
            
            # If version is provided, run following version checks
            # Running following checks for version before validating it because of following reasons =>
            #   1] If we validate version first then it will break incase of invalid version pattern is provided
            #   2] Also no need to validate version if provided version is same or lower than current version
            if version:
                # Check provided version is as per pattern
                Utilities._check_version_for_pattern(version)
                
                status_data = self.utilities.read_status_file(EXECUTE_STATUS_FILE) if os.path.isfile(EXECUTE_STATUS_FILE) else {}
                status_data_phase = status_data.get('phase',None)

                # Compare target_version with current_version
                result = self.utilities.version_compare(self.current_version, version)
                if result == 0 and (not status_data_phase or status_data_phase=='pre-upgrade'):
                    # Target version is same as current version
                    msg = messages.LOWER_OR_SAME_VERSION_PROVIDED.format(self.current_version, version, "same as")
                    self.logger.error("ERROR: {}".format(msg))
                    self.task_utilities.print_justified_text(msg)
                    sys.exit()
                if result == 1:
                    # Target version is lower than current version
                    msg = messages.LOWER_OR_SAME_VERSION_PROVIDED.format(self.current_version, version, "less than")
                    self.logger.error("ERROR: {}".format(msg))
                    self.task_utilities.print_justified_text(msg)
                    sys.exit()
            
            if status_data and status_data_phase !='pre-upgrade':
                Utilities._set_target_version(version)
            else:
                # Validate target_version is supported for upgrade from current_version
                version=Utilities.validate_upgrade_version(self.current_version,version)
            
            if download_packages:
                self.download_packages(version, local_download_directory)
                sys.exit()
            
            # Verify packages are locally downloaded for provided target_version
            Utilities._verify_local_packages_for_version(version)
            
            result = self.utilities.is_command_running(
                'execute', EXECUTE_STATUS_FILE)
            if result[0]:
                err_msg = messages.ERROR_OCCURED.format(
                    "execution of execute command", messages.COMMAND_RUNNING.format(result[1]))
                self.logger.error(err_msg)
                sys.exit(err_msg)
            
            today_date = date.today()
            time_epoch = int(time.time())
            log_file = UPGRADE_LOG_FILE.format(version,today_date,time_epoch)
            log_file_obj = open(log_file,'a')
            original_stdout = sys.stdout
            # Redirect standard output to both CLI and a log file using DualOutputHandler
            sys.stdout = DualOutputHandler(sys.stdout, log_file_obj)
            phase_execution_result = False
            for phase_name in PHASE_MAP_DICT.keys():
                # To handle very first execution of command/phase
                status_data = None
                status_task = None
                status_phase = phase_name
                status = 'started'
                
                if os.path.isfile(EXECUTE_STATUS_FILE):
                    status_data = self.utilities.read_status_file(EXECUTE_STATUS_FILE)
                    if status_data:
                        status_data_task = status_data.get('task',None)
                        status_data_phase = status_data.get('phase',None)
                        status_data_status = status_data.get('status',None)
                        status_data_version = status_data.get('version',None)
                        if status_data_version == version:
                            # This condition will ensure resume of any command for same target version in status file
                            if status_data_phase and phase_name != status_data_phase and status_data_status == 'started':
                                # This will ensure to skip execution of already executed phases
                                print(messages.PHASE_SUCCESSFUL_EXECUTION.format(phase_name.capitalize()))
                                continue

                            if status_data_task and status_data_status == 'started':
                                # To handle resume of execution of command from point of failure
                                status_task = status_data_task
                                status_phase = status_data_phase
                                status = status_data_status
                
                self.utilities.update_command_status(version, current_process_pid, status_phase, status_task, status, EXECUTE_STATUS_FILE)
                phase_execution_result = self._function_code(version, phase_name, 'execute', False, False, EXECUTE_STATUS_FILE)

                if not phase_execution_result:
                    break
            
            # Restore the original standard output
            sys.stdout = original_stdout
            log_file_obj.close()
            self.utilities.update_command_status(version, current_process_pid, None, None, 'ended', EXECUTE_STATUS_FILE)
            if phase_execution_result:
                # Removing contents of workflow directory after successful upgrade
                workflow_path = os.path.join(FRAMEWORK_PATH, "workflow")
                self.utilities.remove_contents(workflow_path)
        except Exception as ex:
            self.logger.exception(messages.ERROR_OCCURED.format("FSR upgrade", ex))
            self.utilities.update_command_status(version, current_process_pid, None, None, 'ended', EXECUTE_STATUS_FILE)
            sys.exit(messages.REFER_LOG)

    def extract_zip(self, zip_path):
        try:
            self.task_utilities.extract_zip(zip_path)
            zip_file = zip_path.split('/')[-1]
            self.task_utilities.print_justified_text(
                messages.EXTRACTED_SUCCESSFULLY.format(zip_file, f"{FRAMEWORK_PATH}/workflow"))
        except Exception as ex:
            self.logger.exception(messages.ERROR_OCCURED.format(
                "extracting files from zip", ex))
            sys.exit(messages.REFER_LOG)

    def create_task(self, phase, task_name, cls_name):
        task_generator = TaskFileGenerator()
        task_generator.verify_arguments(phase, task_name, cls_name)
        task_generator.create_task()

    def create_shell_script(self, phase, shell_script_name):
        shell_file_generator = ShellFileGenerator()
        shell_file_generator.verify_arguments(phase, shell_script_name)
        shell_file_generator.create_script()

    # download the upgrade packages locally 
    def download_packages(self, version, local_download_directory):
        try:
            custom_yum_url = self.config['custom_yum_url'] if 'custom_yum_url' in self.config and self.config['custom_yum_url'] else ""
            if custom_yum_url:
                print(messages.DOWNLOAD_PKG_OFFLINE_REPO)
                sys.exit()
            
            if not local_download_directory:
                local_download_directory=LOCAL_PACKAGES_DOWNLOAD_DIR        

            system_connectors=" ".join(CONNECTORS_LIST)
            os.system(f"sh {FRAMEWORK_PATH}/helpers/download_packages.sh {version} {local_download_directory} '{system_connectors}'")
        except  Exception as ex:
            print(ex) 
            sys.exit("Failed to download the packages ") 
