""" Copyright start
  Copyright (C) 2008 - 2025 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """

import os
import sys
import time
import argparse
import constants
from helpers.logger import Logger
from framework.utility import messages
from framework.framework import Framework
from framework.utility.task_utils import TaskUtilities
from helpers.utils import Utilities
from helpers.custom_help_formatter import CustomHelpFormatter

# Following lines are added to reduce observed delay when upgrade is done using csadm command
# Delay was there because when main.py is called through csadm, python buffer flush was getting stuck till completion of yum update command 
# buffering=1 specifies the buffering policy which is output is buffered until a newline character is encountered 
sys.stdout = os.fdopen(sys.stdout.fileno(), 'w', buffering=1)  # Line buffering
sys.stderr = os.fdopen(sys.stderr.fileno(), 'w', buffering=1)  # Line buffering

class Main(object):
    global_parser_var = None

    def __init__(self) -> None:
        self.logger = Logger.get_logger(__name__)
        self.utils = Utilities()
        
    def _create_log_file_with_version_and_timestamp(self,version) -> None:
        # Create log file with timestamp
        time_epoch = int(time.time())
        log_file_with_timestamp = constants.LOG_FILE_WITH_TIMESTAMP.format(version,time_epoch)
        open(f'{log_file_with_timestamp}', 'w').close()
        os.system(f"chown {constants.FSR_ELEVATE_USER}:{constants.FSR_ELEVATE_USER} {log_file_with_timestamp}")
        os.system(f"chmod 644 {log_file_with_timestamp}")
        os.environ['LOG_FILE_WITH_TIMESTAMP'] = log_file_with_timestamp
        constants.LOG_FILE = log_file_with_timestamp

    def add_check_readiness_parser(self, subparser):
        check_readiness_parser = subparser.add_parser(
            "check-readiness", help="Assess the system's readiness for an upgrade", allow_abbrev=False)
        check_readiness_parser.set_defaults(operation="check-readiness")
        check_readiness_parser.add_argument(
            "--target-version", action="store", type=str, required=True, nargs='?', const="placeholder_value", help="Provide the FortiSOAR version to which the upgrade is supported")

    def add_execute_task_parser(self, subparser):
        execute_task_parser = subparser.add_parser(
            "execute-task", help="Executes a specific task from a phase", allow_abbrev=False)
        execute_task_parser.set_defaults(operation="execute-task")
        execute_task_parser.add_argument(
            "--target-version", dest='target_version', action="store", type=str, required=True, nargs='?', const="placeholder_value", help="Provide the FortiSOAR version to which the upgrade is supported")
        execute_task_parser.add_argument(
            "--phase", action="store", type=str, choices=constants.PHASE_ARGUMENT, required=True, nargs='?', const="placeholder_value", help="Provide the name of the phase")
        execute_task_parser.add_argument(
            "--task-name", dest='task_name', action="store", type=str, required=True, nargs='?', const="placeholder_value", help="Provide the name of the task from a phase to execute")
        execute_task_parser.add_argument(
            "--skip-task-download", action="store_true", required=False, default=True, help=argparse.SUPPRESS)

    def add_execute_phase_parser(self, subparser):
        execute_phase_parser = subparser.add_parser(
            "execute-phase", help="Executes specific phase", allow_abbrev=False)
        execute_phase_parser.set_defaults(operation="execute-phase")
        execute_phase_parser.add_argument(
            "--target-version", dest='target_version', action="store", type=str, required=True, nargs='?', const="placeholder_value", help="Specify the FortiSOAR version you intend to upgrade to")
        execute_phase_parser.add_argument(
            "--phase", action="store", type=str, choices=constants.PHASE_ARGUMENT, required=True, nargs='?', const="placeholder_value", help="Specify phase name")
        execute_phase_parser.add_argument(
            "--skip-task-download", action="store_true", required=False, help=argparse.SUPPRESS)

    def add_execute_parser(self, subparser):
        execute_parser = subparser.add_parser(
            "execute", help="Initiate an upgrade to provided version", allow_abbrev=False)
        execute_parser.set_defaults(operation="execute")
        execute_parser.add_argument(
            "--target-version", dest='target_version' ,action="store", type=str, required=True, nargs='?', const="placeholder_value", help="Specify the FortiSOAR version you intend to upgrade to")
        execute_parser.add_argument(
            "--download-packages", dest='download_packages', action="store_true", required=False, help="Download the packages locally for the specified target version")
        execute_parser.add_argument(
            "--local-download-directory", dest='local_download_directory', action="store", type=str, required=False, nargs='?', const="placeholder_value", help="Specify the absolute path of the local directory where the upgrade packages should be downloaded. The default location is '/opt/cyops/packages'")  

    def add_create_task_parser(self, subparser):
        create_task_parser = subparser.add_parser(
            "create-task", help="Create a task file", allow_abbrev=False)
        create_task_parser.set_defaults(operation="create-task")
        create_task_parser.add_argument(
            "--phase", action="store", type=str, choices=constants.PHASE_ARGUMENT, required=True, nargs='?', const="placeholder_value", help="Specify phase name")
        create_task_parser.add_argument(
            "--task-name", dest='task_name', action="store", type=str, required=True, nargs='?', const="placeholder_value", help="Specify file name")
        create_task_parser.add_argument(
            "--cls-name", dest='cls_name', action="store", type=str, required=True, nargs='?', const="placeholder_value", help="Specify class name")

    def add_create_shell_script_parser(self, subparser):
        create_shell_script_parser = subparser.add_parser(
            "create-shell-script", help="Create a shell file", allow_abbrev=False)
        create_shell_script_parser.set_defaults(
            operation="create-shell-script")
        create_shell_script_parser.add_argument(
            "--phase", action="store", type=str, choices=constants.PHASE_ARGUMENT, required=True, nargs='?', const="placeholder_value", help="Specify phase name")
        create_shell_script_parser.add_argument(
            "--shell-script-name", dest='shell_script_name', action="store", type=str, required=True, nargs='?', const="placeholder_value", help="Specify file name")

    def define_parser(self, parser):
        subparser = parser.add_subparsers()
        self.add_check_readiness_parser(subparser)
        self.add_execute_task_parser(subparser)
        self.add_execute_phase_parser(subparser)
        self.add_execute_parser(subparser)
        self.add_create_task_parser(subparser)
        self.add_create_shell_script_parser(subparser)

        for name, subp in subparser.choices.items():
            subp._positionals.title = 'subcommand options are'
            subp._optionals.title = 'subcommand options are'

    def main(self):
        parser = argparse.ArgumentParser(prog="csadm upgrade", formatter_class=CustomHelpFormatter)
        parser._positionals.title = "subcommand options are"
        parser._optionals.title = "subcommand options are"
        self.define_parser(parser)

        global global_parser_var
        global_parser_var = parser

        args = parser.parse_args()
        if 'operation' not in dir(args):
            parser.print_help(sys.stderr)
            sys.exit(1)

        current_version = TaskUtilities.get_current_version()
        version = args.target_version if hasattr(args,'target_version') and args.target_version else current_version
        
        # To avoid creating a log file for the below mentioned commands, as no logs were created for these commands, an unnecessary 0-byte log file was being created.
        log_file_skip_list = ['create-task', 'create-shell-script']
        if not any(arg in sys.argv for arg in log_file_skip_list) and ('--download-packages' not in sys.argv):
            # Create log file with version and timestamp in name
            self._create_log_file_with_version_and_timestamp(version)
        
        # Creating framework object after creation of log file with version and timestamp in log file name
        self.framework = Framework()
        
        # Check for Docker, as the upgrade on Docker has a different path.
        command = sys.argv[1] if len(sys.argv) > 2 and sys.argv[1]!='execute' else ""
        if self.utils.is_docker():
            if command:
                print(messages.DOCKER_INSTANCE_MSG.format(constants.TEXT_COLOR["RED"],command,constants.TEXT_COLOR["RESET"]))
            else:
                print(messages.DOCKER_INSTANCE_UPGRADE_MSG.format(constants.TEXT_COLOR["RED"],constants.TEXT_COLOR["RESET"]))
            sys.exit()
        
        if str(args.operation) == "check-readiness":
            self.framework.check_readiness(args.target_version)

        if str(args.operation) == "execute-task":
            self.framework.execute_task(
                args.target_version, args.phase, args.task_name, args.skip_task_download)

        if str(args.operation) == "execute-phase":
            self.framework.execute_phase(
                args.target_version, args.phase, args.skip_task_download)

        if str(args.operation) == "execute":
            self.framework.execute(args.target_version, args.download_packages, args.local_download_directory)

        if str(args.operation) == "create-task":
            self.framework.create_task(
                args.phase, args.task_name, args.cls_name)

        if str(args.operation) == "create-shell-script":
            self.framework.create_shell_script(
                args.phase, args.shell_script_name)

class_object = Main()
class_object.main()
